import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;
import java.awt.Font;
import java.awt.image.BufferedImage;
import java.awt.FontMetrics;

public class Button extends Actor
{
    int width;
    int height;
    int startx;
    int starty;
    int endx;
    int endy;
    String text;
    public boolean isClicked = false;
    public boolean isHovering = false;
    //Font defaultFont = new Font("Times New Roman", Font.BOLD, 24);
    Font defaultFont;
    Color buttonColor = new Color(150, 150, 150);
    Color outlineColor = new Color(50, 50, 50);
    Color textColor = new Color(255, 255, 255);
    Color highlightColor = new Color(180, 180, 180);
    
    public Button(int width, int height, String text)
    {
        this.height = height;
        this.width = width;
        this.text = text;
        defaultFont = new Font("Tahoma", Font.BOLD, 24);
        paint();
    }
    public Button(int width, int height, String text, int textSize)
    {
        this.height = height;
        this.width = width;
        this.text = text;
        defaultFont = new Font("Tahoma", Font.BOLD, textSize);
        paint();
    }
    
    /**
     * Sets the colors used in the button
     * @param buttonColor The main fill color of the button when not being hovered over
     * @param outlineColor The outline of the button
     * @param textColor The color of the text displayed on the button
     * @param highlightColor The main fill color of the button when being hovered over
     */
    public void setColors(Color buttonColor, Color outlineColor, Color textColor, Color highlightColor) {
        this.buttonColor = buttonColor;
        this.outlineColor = outlineColor;
        this.textColor = textColor;
        this.highlightColor = highlightColor;
    }
    
    /**
     * Called every time step, paints the background
     */
    public void paint()
    {
        GreenfootImage myImage = new GreenfootImage(width, height);
        if(!isHovering)
            myImage.setColor(buttonColor);
        else
            myImage.setColor(highlightColor);
        myImage.fill();
        myImage.setColor(outlineColor);
        myImage.drawRect(0,0, width-1, height-1);
        myImage.setColor(textColor);
        myImage.setFont(defaultFont);
        String stringToDraw = text;
        int nheight = myImage.getHeight();
        int nwidth = myImage.getWidth();
        int strWidth = stringWidth(stringToDraw, defaultFont);
        int strHeight = stringHeight(stringToDraw, defaultFont);
        GreenfootImage drawImage = new GreenfootImage(strWidth, (int)(1.4 * strHeight));
        drawImage.setFont(defaultFont);
        drawImage.setColor(textColor);
        drawImage.drawString(stringToDraw, 0, strHeight);
        int drawHeight = (int)((nheight - (int)(1.3 * strHeight))/2.0);
        myImage.drawImage(drawImage, (int)((nwidth - strWidth)/2.0), drawHeight);
        
        setImage(myImage);
    }
    
    /**
     * Used to determine where to horizontally center the button text
     * @param str The string to find the width of
     * @param font The font to use when finding the width of the string
     * @return stringWidth The width of the string in pixels using the given font
     */
    public int stringWidth(String str, Font font)
    {
        BufferedImage img = new BufferedImage(1, 1, BufferedImage.TYPE_INT_ARGB);
        FontMetrics fm = img.getGraphics().getFontMetrics(font);
        int width = fm.stringWidth(str);
        return width;
    }
    
    /**
     * Used to determine where to vertically center the button text
     * @param str The string to find the average height of
     * @param font The font to use when finding the average height of the string
     * @return stringWidth The height of the string in pixels using the given font
     */
    public int stringHeight(String str, Font font)
    {
        BufferedImage img = new BufferedImage(1, 1, BufferedImage.TYPE_INT_ARGB);
        FontMetrics fm = img.getGraphics().getFontMetrics(font);
        int height = fm.getHeight();
        return height;
    }
    
    /**
     * Called every time step, determines whether
     */
    
    public void act() 
    {
        startx = getX() - (int)(width/2.0);
        endx = getX() + (int)(width/2.0);
        starty = getY() - (int)(height/2.0);
        endy = getY() + (int)(height/2.0);
        paint();
        MouseInfo mi = Greenfoot.getMouseInfo();
        if(mi!=null) {
            if((mi.getX() < endx) && (mi.getX() > startx) && (mi.getY() < endy) && (mi.getY() > starty)) {
                isHovering = true;
            }
            else {
                isHovering = false;
            }
        }
        
        
        if(Greenfoot.mouseClicked(this)) { isClicked = true; }
        else { isClicked = false; }
    }    
}
